library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
library rfsm;
use rfsm.core.all;

entity heron is
  port(
        h: in std_logic;
        start: in integer;
        u: in real;
        rdy: out integer;
        r: out real;
        rst: in std_logic
        );
end heron;

architecture RTL of heron is
  type t_state is ( Idle, Iter );
  signal state: t_state;
  signal a: real;
  signal x: real;
  signal n: integer;
begin
  process(rst, h)
  begin
    if ( rst='1' ) then
      state <= Idle;
      rdy <= 1;
    elsif rising_edge(h) then 
      case state is
      when Iter =>
        if ( n<8 ) then
          x <= (x+(a/x))/2.0;
          n <= n+1;
        elsif  ( n = 8 ) then
          r <= x;
          rdy <= 1;
          state <= Idle;
        end if;
      when Idle =>
        if ( start = 1 ) then
          a <= u;
          x <= u;
          rdy <= 0;
          n <= 0;
          state <= Iter;
        end if;
    end case;
    end if;
  end process;
end RTL;
