library ieee;
use ieee.std_logic_1164.all;	   
use ieee.numeric_std.all;
library rfsm;
use rfsm.core.all;

entity tb is
end tb;

architecture Bench of tb is

component heron 
  port(
        h: in std_logic;
        start: in integer;
        u: in real;
        rdy: out integer;
        r: out real;
        rst: in std_logic
        );
end component;

signal U: real;
signal Start: integer;
signal H: std_logic;
signal R: real;
signal Rdy: integer;
signal rst: std_logic;

begin

  inp_U: process
    type t_vc is record date: time; val: real; end record;
    type t_vcs is array ( 0 to 0 ) of t_vc;
    constant vcs : t_vcs := ( others => (5 ns,2.0) );
    variable i : natural := 0;
    variable t : time := 0 ns;
    begin
      for i in 0 to 0 loop
        wait for vcs(i).date-t;
        U <= vcs(i).val;
        t := vcs(i).date;
      end loop;
      wait;
  end process;
  inp_Start: process
    type t_vc is record date: time; val: integer; end record;
    type t_vcs is array ( 0 to 2 ) of t_vc;
    constant vcs : t_vcs := ( (0 ns,0), (25 ns,1), (35 ns,0) );
    variable i : natural := 0;
    variable t : time := 0 ns;
    begin
      for i in 0 to 2 loop
        wait for vcs(i).date-t;
        Start <= vcs(i).val;
        t := vcs(i).date;
      end loop;
      wait;
  end process;
  inp_H: process
    type t_periodic is record period: time; t1: time; t2: time; end record;
    constant periodic : t_periodic := ( 9 ns, 10 ns, 200 ns );
    variable t : time := 0 ns;
    begin
      H <= '0';
      wait for periodic.t1;
      notify_ev(H,1 ns);
      while ( t < periodic.t2 ) loop
        wait for periodic.period;
        notify_ev(H,1 ns);
        t := t + periodic.period;
      end loop;
      wait;
  end process;

  U0: Heron port map(H,Start,U,Rdy,R,rst);

  process

  begin
    rst <= '1';
    wait for 1 ns;
    rst <= '0';
    wait for 200 ns;
    wait;

  end process;
end Bench;
