library ieee;
use ieee.std_logic_1164.all;	   
use ieee.numeric_std.all;
library rfsm;
use rfsm.core.all;

entity tb is
end tb;

architecture Bench of tb is

component s 
  port(
        h: in std_logic;
        start: in boolean;
        x: in integer;
        rdy: out boolean;
        r: out integer;
        rst: in std_logic
        );
end component;

signal x: integer;
signal Start: boolean;
signal H: std_logic;
signal r: integer;
signal Rdy: boolean;
signal rst: std_logic;

begin

  inp_x: process
    type t_vc is record date: time; val: integer; end record;
    type t_vcs is array ( 0 to 3 ) of t_vc;
    constant vcs : t_vcs := ( (25 ns,100), (245 ns,200), (465 ns,0), (605 ns,65535) );
    variable i : natural := 0;
    variable t : time := 0 ns;
    begin
      for i in 0 to 3 loop
        wait for vcs(i).date-t;
        x <= vcs(i).val;
        t := vcs(i).date;
      end loop;
      wait;
  end process;
  inp_Start: process
    type t_vc is record date: time; val: boolean; end record;
    type t_vcs is array ( 0 to 8 ) of t_vc;
    constant vcs : t_vcs := ( (0 ns,false), (35 ns,true), (45 ns,false), (255 ns,true), (265 ns,false), (475 ns,true), (485 ns,false), (615 ns,true), (625 ns,false) );
    variable i : natural := 0;
    variable t : time := 0 ns;
    begin
      for i in 0 to 8 loop
        wait for vcs(i).date-t;
        Start <= vcs(i).val;
        t := vcs(i).date;
      end loop;
      wait;
  end process;
  inp_H: process
    type t_periodic is record period: time; t1: time; t2: time; end record;
    constant periodic : t_periodic := ( 9 ns, 0 ns, 1000 ns );
    variable t : time := 0 ns;
    begin
      H <= '0';
      wait for periodic.t1;
      notify_ev(H,1 ns);
      while ( t < periodic.t2 ) loop
        wait for periodic.period;
        notify_ev(H,1 ns);
        t := t + periodic.period;
      end loop;
      wait;
  end process;

  U0: S port map(H,Start,x,Rdy,r,rst);

  process

  begin
    rst <= '1';
    wait for 1 ns;
    rst <= '0';
    wait for 1000 ns;
    wait;

  end process;
end Bench;
