library ieee;
use ieee.std_logic_1164.all;	   
use ieee.numeric_std.all;
library rfsm;
use rfsm.core.all;

entity tb is
end tb;

architecture Bench of tb is

component c1 
  port(
        h: in std_logic;
        start: in std_logic;
        x0: in integer;
        rdy: out std_logic;
        n: out integer;
        rst: in std_logic
        );
end component;

signal x0: integer;
signal Start: std_logic;
signal H: std_logic;
signal n: integer;
signal Rdy: std_logic;
signal rst: std_logic;

begin

  inp_x0: process
    type t_vc is record date: time; val: integer; end record;
    type t_vcs is array ( 0 to 0 ) of t_vc;
    constant vcs : t_vcs := ( others => (0 ns,10) );
    variable i : natural := 0;
    variable t : time := 0 ns;
    begin
      for i in 0 to 0 loop
        wait for vcs(i).date-t;
        x0 <= vcs(i).val;
        t := vcs(i).date;
      end loop;
      wait;
  end process;
  inp_Start: process
    type t_vc is record date: time; val: std_logic; end record;
    type t_vcs is array ( 0 to 2 ) of t_vc;
    constant vcs : t_vcs := ( (0 ns,'0'), (25 ns,'1'), (35 ns,'0') );
    variable i : natural := 0;
    variable t : time := 0 ns;
    begin
      for i in 0 to 2 loop
        wait for vcs(i).date-t;
        Start <= vcs(i).val;
        t := vcs(i).date;
      end loop;
      wait;
  end process;
  inp_H: process
    type t_periodic is record period: time; t1: time; t2: time; end record;
    constant periodic : t_periodic := ( 9 ns, 10 ns, 200 ns );
    variable t : time := 0 ns;
    begin
      H <= '0';
      wait for periodic.t1;
      notify_ev(H,1 ns);
      while ( t < periodic.t2 ) loop
        wait for periodic.period;
        notify_ev(H,1 ns);
        t := t + periodic.period;
      end loop;
      wait;
  end process;

  U0: C1 port map(H,Start,x0,Rdy,n,rst);

  process

  begin
    rst <= '1';
    wait for 1 ns;
    rst <= '0';
    wait for 100 ns;
    wait;

  end process;
end Bench;
