(* Two independent (no synchronisation) counters *)

open Fsm
open Expr
open Types
open Stim

let mk_counter i maxv =
  let x = "x" ^ string_of_int i in
  let n = string_of_int maxv in
  Fsm.create
    ~name:("C" ^ string_of_int i)
    ~params:[]
    ~states:["A";"B"]
    ~inps:["H", TyEvent]
    ~outps:[]
    ~vars:[x, TyInt (Some (0,maxv))]
    ~trans:[
       ("A", "H", x^":=1", "B");  
       ("B", "H; "^x^"<"^n, x^":="^x^"+1", "B");  
       ("B", "H; "^x^"="^n, x^":=0", "A")
       ]
    ~resolve:None
    ~itrans:("A",x^":=0")

let a1 = mk_counter 1 2
let a2 = mk_counter 2 3

let _ = Fsm.dot_output "a1.dot" a1
let _ = Fsm.dot_output "a2.dot" a2

let ss = mk_per_event "H" 10 10 100

(* let ctx, rs = Simul.run [a1; a2]  ss *)
(* let _ = List.iter Simul.dump_reaction rs *)
(* let _ = Vcd.output ctx "counters.vcd" rs *)

let p = Fsm.product Fsm.Blocking "P" a1 a2
let _ = Fsm.dot_output "p.dot" p

let _ = Fsm.text_output a1
let _ = Fsm.text_output a2
let _ = Fsm.text_output p

let ctx, rs = Simul.run [a1; a2; p]  ss
let _ = Vcd.output ctx "counters.vcd" rs
