open Rfsm
open Expr
open Types
open Action
  
let t_ctl = TyEnum ["Ok"; "NOk"]
let t_cmd = TyEnum ["On"; "Off"]
          
let foo = Fsm.build_model
  ~name:"foo"
  ~states:["Stopped"; "Running"]
  ~params:[]
  ~ios:[
    IO_In, "h", TyEvent;
    IO_In, "en", t_ctl;
    IO_Out, "o", t_cmd;
    ]
  ~vars:["v", TyBool]
  ~trans:[
    ("Stopped", ("h",[EVar "en","=",EEnum "Ok"]), [Assign ("o",EEnum "On"); Assign ("v", EBool true)], "Running");
    ("Running", ("h",[]), [Assign ("o",EEnum "Off")], "Stopped");
    ]
  ~itrans:("Stopped",[])

let h = Fsm.GInp ("H", TyEvent, Periodic (10,10,110))
let en = Fsm.GInp ("En", t_ctl, ValueChange [0,Val_enum "NOk"; 40,Val_enum "Ok"])
let o = Fsm.GOutp ("Out", t_cmd)

let f = Fsm.build_instance ~name:"c1" ~model:foo ~params:[] ~ios:[h;en;o]

let p = Comp.build_composite "f" [f]

(* let _ = Comp.dot_output ~with_insts:true ~with_models:true "./dot" p *)
let _ = Comp.dot_output "./dot" p

let c, rs = Simul.run p
let _ = List.iter Simul.dump_reaction rs

